#!/bin/sh
# install.sh: nuhed installation script
#
#


PKGLIST=""
NUHEDDIR=/usr/local/etc/nuhed


PKGLIST=""
BASEDIR="/usr/local"
PROG="nuhed"
NUHEDD_DIR="$BASEDIR/$PROG"

#zlib
RPMS="python3"
DPKGS="python3 python3-openssl"


#zlib
#RPMS8="logrotate iptables gcc make  aclocal autoconf automake openssl-devel pcre-devel openssl openssl-utils"
RPMS8="logrotate iptables gcc make  autoconf automake openssl-devel pcre-devel openssl"
RPMS9="logrotate iptables-nft gcc make  autoconf automake openssl-devel pcre-devel openssl"
DPKGS="logrotate iptables gcc make automake autoconf libssl-dev libpcre3-dev"


check_packages () {
  OS=$1
  PKGS=$2

  for PKG in $PKGS
  do
    if [ "$OS" = "debian" ];then
       sudo dpkg -s $PKG > /dev/null 2>/dev/null
    elif [ "$OS" = "rhel" ];then
       sudo rpm -q $PKG > /dev/null 2>/dev/null
    else
       echo "Invalid OS version"
       exit 1
    fi

    if [ "$?" -eq "1" ];then
         PKGLIST="$PKGLIST $PKG"
    fi
  done

}


# Ansi color code variables
red="\e[0;91m"
expand_bg="\e[K"
red_bg="\e[0;101m${expand_bg}"
green_bg="\e[0;102m${expand_bg}"
green="\e[0;92m"
white="\e[0;97m"
bold="\e[1m"
uline="\e[4m"
reset="\e[0m"

CONFIG_FILE=""
CONFIG_FILE_CURDIR="./nuhed_install.conf"
CONFIG_FILE_BACKUP="../nuhed_install.conf"
CONFIG_STATUS="f"

if [ -f $CONFIG_FILE_CURDIR ];then
   if [ -z "$(grep \< $CONFIG_FILE_CURDIR)" ];then
      CONFIG_FILE=$CONFIG_FILE_CURDIR
   fi
fi

if [ -z "$CONFIG_FILE" ];then
   if [ -f $CONFIG_FILE_BACKUP ];then
      if [ -z "$(grep \< $CONFIG_FILE_BACKUP)" ];then
         CONFIG_FILE=$CONFIG_FILE_BACKUP
      fi
   fi
fi

if [ -z "$CONFIG_FILE"  ];then
   echo "Error: Configuration file not found from  $CONFIG_FILE_CURDIR or $CONFIG_FILE_BACKUP or sensor information not set";
   exit 1
fi


NODEMGR="$(cat $CONFIG_FILE | grep NODEMGR | cut -d: -f2)"
SENSOR_HOSTNAME=$(cat $CONFIG_FILE | grep SENSOR_HOSTNAME | cut -d: -f2)
SENSOR_COUNTRY_CODE=$(cat $CONFIG_FILE | grep SENSOR_COUNTRY_CODE | cut -d: -f2)
SENSOR_ORG=$(cat $CONFIG_FILE  | grep SENSOR_ORG | cut -d: -f2)


SUID=$(id -u)
echo -n "Checking for sudo rights "
# check for sudo rights
if [ "$SUID" -ne "0" ];then
  echo  "${red}${bold}FAIL${reset}"
  echo
  echo  "${red}${bold}Installation requires sudo rights (sudo ./install.sh)${reset}"
  exit 1
else
  echo  "${green}${bold}OK${reset}"
fi
echo


# Installing required packages
if [ -f /etc/redhat-release ];then

    OS="rhel"
    VER9=$(cat /etc/redhat-release | grep 9) 
    if [ -n "$VER9" ];then
      RPMS="$RPMS9"
    else 
      RPMS="$RPMS8"
    fi	    
    check_packages "$OS" "$RPMS"
    echo "Following packaged needs to be installed: $PKGLIST"

elif [ -f /etc/debian_version ];then
    OS="debian"
    check_packages "$OS" "$DPKGS"
    echo "Following packaged needs to be installed: $PKGLIST"

else
   echo "Unkown OS version"
   exit 1
fi
echo

if [ ! -z "$PKGLIST" ];then
  echo -n "Install packages [Y/n] ? "
  read var

  if [ "$var" = "y" ] || [ "$var" = "y" ] || [ "$var" = "" ];then
    if [ "$OS" = "rhel" ];then
      sudo yum install -y $PKGLIST
    else
      sudo apt -qq install -y $PKGLIST
      if [ ! -f /sbin/iptables ];then
         ln -s /usr/sbin/iptables /sbin/iptables
      fi
    fi
  fi
fi



# Create nuheddir
if [ ! -d "$NUHEDDIR" ];then
  mkdir  "$NUHEDDIR"
fi

echo
# Check for wget
echo -n "Checking if wget exists "
if [ -f /usr/bin/wget ] || [ -f /usr/local/bin/wget ];then
  echo  "${green}${bold}OK${reset}"
else
  echo  "${red}${bold}FAIL${reset}"
fi
echo 

# Download version information
wget -q http://www.nuhedman.com/download/versions -O ./versions_ 2>/dev/null 1>/dev/null
if [ $? -ne 0 ];then
    echo "Warning: Unable to to download latest versions file, using default"
    echo
else
    mv ./versions_ versions
fi

if [ ! -d ../nuhed ] && [  -f ./versions ];then
  NUHED_VER=$(cat versions | grep nuhed: | cut -d: -f2)

  # Download source code
  if [ ! -f ./${PROG}-${NUHED_VER}.tar.gz ];then
    echo -n "Downloading ${PROG}-${NUHED_VER} source code files "
    wget -q http://www.nuhedman.com/download/${PROG}-${NUHED_VER}.tar.gz -O ./${PROG}-${NUHED_VER}.tar.gz_ 2>/dev/null 1>/dev/null
    if [ "$?"="0" ];then
       mv ./${PROG}-${NUHED_VER}.tar.gz_ ./${PROG}-${NUHED_VER}.tar.gz
    else
      echo " Error: Unable to download ${PROG}-${NUHED_VER} source code files "
      echo " Manually download from http://www.nuhedman.com/download/${PROG}-${NUHED_VER}.tar.gz  "
      exit 1
    fi
  fi

  echo -n "Checking if nuhed.tar.gz exists "
  if [ -f ./${PROG}-${NUHED_VER}.tar.gz ];then
    echo  "${green}${bold}OK${reset}"
  else
    echo  "${red}${bold}FAIL${reset}"
    exit 1
  fi

  echo 
  # unpack installation package
  echo -n "Unpacking source files "
  gzip -dc ./${PROG}-${NUHED_VER}.tar.gz | tar -xvf -  > /dev/null 2>/dev/null
  if [ -d ./nuhed ];then
    echo  "${green}${bold}OK${reset}"
  else
    echo  "${red}${bold}FAIL${reset}"
    exit 1
  fi
  echo

fi

# stop running nuhed process
echo -n "Stopping running nuhed instance "

WAITSEC=0
NUHEPID=""
while true;
do
  NUPID=""
  if [ -f /var/run/nuhed.pid ];then
     NUPID=$(cat /var/run/nuhed.pid)
  fi

  if [ -n "$NUPID" ];then
     kill $NUPID 2>/dev/null 1>/dev/null
  else
    break
  fi

  if [ $WAITSEC -ge 15 ];then
    break
  fi

  sleep 1
  WAITSEC=$(($WAITSEC + 1))
 echo -n ". "
done

if [ -z "$NUPID" ];then
  echo  "${green}${bold}OK${reset}"
  echo
else

  echo  "${red}${bold}FAIL${reset}"
  echo 
  echo "Failed stop running nuhed process PID $NUPID. Stop manually (kill -9) or/and remove /var/run/nuhed.pid if exists."
  exit 1
fi


echo 
if [ -d ./nuhed ];then
  NUHED_INSTALL_DIR=./nuhed
else
  NUHED_INSTALL_DIR=../nuhed
fi
# keep only general rules
cp  $NUHED_INSTALL_DIR/nuhed.conf $NUHED_INSTALL_DIR/nuhed.conf_
cat $NUHED_INSTALL_DIR/nuhed.conf_ | grep -v openssh. | grep -v iptables. | grep -v smtp. | grep -v email. > $NUHED_INSTALL_DIR/nuhed.conf

# Configure sources
cd $NUHED_INSTALL_DIR
echo -n "Configuring nuhed sources "
./configure --enable-nodemgr > ../configure.log
if [ -f ./config.h ];then
  echo  "${green}${bold}OK${reset}"
else
  echo   "${red}${bold}FAIL${reset}"
  echo 
  echo  "${red}${bold}Configure failed, see configure.log for more details.${reset}"
  exit 1
fi
echo 

# build 
echo -n "Building nuhed "
make 2> ../compile.log 1>/dev/null

if [ "$?" -eq "2" ];then
  echo   "${red}${bold}FAIL${reset}"
  echo 
  echo  "${red}${bold}Compile failed, see compile.log for more details.${reset}"
  exit 1
else
  echo  "${green}${bold}OK${reset}"
fi
echo 


# install nuhed
echo
echo "Installing nuhed with 'make install' "
make install 2>/dev/null 1>/dev/null
cd -

if [ "$?" -eq "2" ];then
  echo   "${red}${bold}FAIL${reset}"
  echo 
  echo  "${red}${bold}Make install failed, see install.log for more details.${reset}"
  exit 1
else
  echo  "${green}${bold}OK${reset}"
fi
echo 


# setup configuration
echo -n "Configure nodemanager (nuhed.conf) "
sudo cat /usr/local/etc/nuhed/nuhed.conf | grep -v NodemanagerAddr | sed -e 's/Standalone 1/Standalone 0 /g' > /usr/local/etc/nuhed/nuhed.conf_
echo "NodemanagerAddr $NODEMGR" >> /usr/local/etc/nuhed/nuhed.conf_
sudo mv /usr/local/etc/nuhed/nuhed.conf_ /usr/local/etc/nuhed/nuhed.conf

if [ ! -f $NUHEDDIR/nuhed.conf ];then
  echo   "${red}${bold}FAIL${reset}"
  echo 
  echo  "${red}${bold}Make install failed, see install.log for more details.${reset}"
  exit 1
else
  echo  "${green}${bold}OK${reset}"
fi
echo 

# create client certificate
echo -n "Create client certificate "
sudo openssl req -x509 -newkey rsa:2048 -keyout /usr/local/etc/nuhed/nuhed.key -out /usr/local/etc/nuhed/nuhed.crt -nodes -days 3600 -subj "/C=$SENSOR_COUNTRY_CODE/ST=/L=/O=$SENSOR_ORG LTD/OU=/CN=$SENSOR_HOSTNAME/emailAddress="

if [ ! -f /usr/local/etc/nuhed/nuhed.crt ];then
  echo   "${red}${bold}FAIL${reset}"
  echo 
  echo  "${red}${bold}Failed to create client certificate /usr/local/etc/nuhed/nuhed.crt${reset}"
  exit 1
else
  echo  "${green}${bold}OK${reset}"
fi
echo 

echo -n "Add server certificate "
sudo sh -c 'cat sserver.crt >> /usr/local/etc/nuhed/CAnuhe.pem'
if [ ! -f $NUHEDDIR/CAnuhe.pem ];then
  echo   "${red}${bold}FAIL${reset}"
  echo 
  echo  "${red}${bold}Failed to add server certificate /usr/local/etc/nuhed/CAnuhe.pem${reset}"
  exit 1
else 
  echo  "${green}${bold}OK${reset}"
fi
echo 


touch /usr/local/etc/nuhed/nrules.asc
echo -n "Starting nuhed services /usr/local/sbin/nuhed ... "
sudo /usr/local/sbin/nuhed
if [ "$?" -eq "0" ];then
  echo  "${green}${bold}OK${reset}"
else
  echo   "${red}${bold}FAIL${reset}"
  echo 
  echo  "${red}${bold}Make install failed, see install.log for more details.${reset}"
  exit 1
fi
echo 


echo 
echo "Sensor's client certificate, update client details in NuhedManager"
echo 
cat $NUHEDDIR/nuhed.crt

echo
WHITELISTFILE=$(cat $NUHED_INSTALL_DIR/nuhed.conf | grep  WhitelistFile | awk '{ print $2}')
echo "Actions allowd to run by nuhed:"
if [ -f $WHITELISTFILE ];then
  cat $WHITELISTFILE
else
  echo "  iptables"
  echo "iptables" > $WHITELISTFILE
fi

echo
echo "Edit $WHITELISTFILE to allow commands to be run by sensor."
echo "Use * for any."

echo
echo
if [ -d /etc/systemd/system ];then
  sudo cp -f nuhed.service /etc/systemd/system/nuhed.service
  sudo chmod 664 /etc/systemd/system/nuhed.service
  sudo systemctl daemon-reload
  sudo systemctl enable nuhed 2>/dev/null
fi

exit 0
